package com.chelvc.cloud.admin.controller;

import javax.validation.Valid;
import javax.validation.constraints.Min;

import com.chelvc.cloud.admin.copier.EmployeeCopier;
import com.chelvc.cloud.admin.vo.PersonalVO;
import com.chelvc.cloud.user.client.EmployeeClient;
import com.chelvc.cloud.user.client.dto.EmployeeDTO;
import com.chelvc.cloud.user.client.dto.EmployeeDetailDTO;
import com.chelvc.cloud.user.client.param.EmployeeModifyParam;
import com.chelvc.cloud.user.client.param.EmployeePagingParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 员工接口
 *
 * @author Woody
 * @date 2021/10/20
 */
@Validated
@RestController
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EmployeeController {
    private final EmployeeClient employeeClient;

    /**
     * 新增员工
     *
     * @param param 新增参数
     * @return 员工主键
     */
    @PostMapping("/employee")
    public Long addEmployee(@RequestBody @Valid EmployeeModifyParam param) {
        return this.employeeClient.addEmployee(param);
    }

    /**
     * 修改员工
     *
     * @param id    员工主键
     * @param param 修改参数
     */
    @PutMapping("/employee/{id}")
    public void updateEmployee(@PathVariable("id") @Min(value = 1, message = "员工主键不能小于1") Long id,
                               @RequestBody @Valid EmployeeModifyParam param) {
        this.employeeClient.updateEmployee(id, param);
    }

    /**
     * 获取当前员工个人信息
     *
     * @return 员工个人信息
     */
    @GetMapping("/employee/mine")
    public PersonalVO getPersonal() {
        return EmployeeCopier.INSTANCE.copying(this.employeeClient.getPersonal());
    }

    /**
     * 获取员工详情
     *
     * @param id 员工主键
     * @return 员工详情
     */
    @GetMapping("/employee/{id}/detail")
    public EmployeeDetailDTO getEmployeeDetail(
            @PathVariable("id") @Min(value = 1, message = "员工主键不能小于1") Long id) {
        return this.employeeClient.getEmployeeDetail(id);
    }

    /**
     * 查询员工分页
     *
     * @param param 查询参数
     * @return 员工分页信息
     */
    @GetMapping("/employee/paging")
    public Pagination<EmployeeDTO> getEmployeePaging(@Valid EmployeePagingParam param) {
        return this.employeeClient.getEmployeePaging(param);
    }
}
